@@ -48,10 +48,6 @@
 #define READ   0
 #define WRITE 1
 
-#define ADDED_DISK_OK	1
-#define ADDED_DISK_NOK	2
-#define ADDED_DISK_DELETE	3
-
 static char *local_foldername;
 static char *local_diskname;
 static gboolean flag_to_umount;
@@ -86,7 +82,11 @@ spawn_proc (gchar * path, gint narg, gchar ** arg)
       for (i = 0; i < narg; i++)
 	arg2[i] = arg[i];
       arg2[narg] = NULL;
-
+#if !defined(__DEBUG__)
+      close (0);
+      close (1);
+      close (2);
+#endif
       return execvp (arg2[0], arg2);
     }
   else
@@ -336,7 +336,8 @@ do_not_forget_to_mount (gboolean gb)
 void
 set_added_disk (gint gb)
 {
-  added_disk_is_ok = gb;
+  if ((gb == ADDED_DISK_UNDEF) || (added_disk_is_ok == ADDED_DISK_UNDEF))
+    added_disk_is_ok = gb;
 }
 
 void
@@ -350,7 +351,8 @@ gint my_timeout (gpointer data)
   FOLDER *racine = data;
 /*  FILE_DATA *fd;*/
   display_message_if_necessary ();
-  if (get_thread_status () == THREAD_SCAN_RUNNING_ENDED)
+  if ((get_thread_status () == THREAD_SCAN_RUNNING_ENDED)
+      && (added_disk_is_ok != ADDED_DISK_UNDEF))
     {
       if (added_disk_is_ok == ADDED_DISK_OK)
 	{
@@ -367,8 +369,10 @@ gint my_timeout (gpointer data)
 	{
 	  suppress_dir ((GNode *) local_gn);
 	}
-      g_free (local_diskname);
-      g_free (local_foldername);
+      if (local_diskname)
+	g_free (local_diskname);
+      if (local_foldername)
+	g_free (local_foldername);
 /************** If umount and eject have to be in the main thread, remove the comments ********/
 //umount_and_eject_if_necessary();
 /*********************************************************************************************/
@@ -386,7 +390,7 @@ gint my_timeout (gpointer data)
   else
     {
       progress_timeout (GINT_TO_POINTER (GNOMEAPPBAR_MAIN));
-      gtk_timeout_add (500, my_timeout, racine);
+      gtk_timeout_add (PROGRESS_BAR_TIMEOUT, my_timeout, racine);
     }
   return (0);
 }
@@ -394,7 +398,8 @@ gint my_timeout (gpointer data)
 void
 set_local_disk_name (gchar * s)
 {
-  local_diskname = g_strdup (s);
+  if (added_disk_is_ok == ADDED_DISK_UNDEF)
+    local_diskname = g_strdup (s);
   return;
 }
 
@@ -658,15 +663,21 @@ create_folder_tree (FOLDER * racine, GNode * parent, const gchar * path,
 			vfs_scan_nok = TRUE;
 		    }
 		}
+	      else
+		{
+		  if (mime)
+		    g_free (mime);
+		}
 	    }
 	}
       free (file);
 #if !defined(USE_PTHREADS)
       gui_update ();
 #endif
-
     }
   closedir (dir);
+  g_string_free (my_path, TRUE);
+
   while (new_dirs)
     {
       gn = (GNode *) new_dirs->data;
@@ -741,8 +752,9 @@ start_thread_scan (void *arg)
 	      if (my_config->warn_mount)
 		{
 		  /* FIXME: Error Message asking if we should continue */
-		  printf
-		    ("FIXME: Add error message for cd already mounted\n");
+		  set_message_to_display (g_string_new
+					  (_
+					   ("The CD player is already mounted.")));
 		}
 	    }
 	  else
@@ -750,10 +762,12 @@ start_thread_scan (void *arg)
 	      if (mount_disk (my_config) == FALSE)
 		{
 		  /* Display Error */
-		  printf
-		    ("FIXME: Add error message for error in mounting CD\n");
+		  set_message_to_display (g_string_new
+					  (_
+					   ("Could not mount the device. Is the CD player empty?")));
 		  added_disk_is_ok = ADDED_DISK_NOK;
 		  set_thread_status_to_ended ();
+		  vfs_scan_nok = FALSE;
 #if defined(__DEBUG__)
 		  GML_log (2, "Exiting start_thread_scan(): error\n");
 #endif
