@@ -98,9 +98,6 @@ enable_gnome_button_bar (GtkWidget * window, gpointer data)
   memcpy (&(toolbar[9]), &(general_toolbar[12]), sizeof (GnomeUIInfo));
 
   gnome_app_create_toolbar_with_data (GNOME_APP (window), toolbar, NULL);
-#if defined(__DEBUG__)
-  GML_malloc_forget (toolbar);
-#endif
 }
 
 void
@@ -124,7 +121,7 @@ cb_open_recent (GtkWidget * w, gpointer data)
 {
   FOLDER *racine;
   gchar *p;
-  gchar *filename = g_strdup (data);
+  gchar *filename = data;
 
   if (get_thread_status () == THREAD_SCAN_RUNNING_NO)
     {
@@ -152,6 +149,10 @@ cb_open_recent (GtkWidget * w, gpointer data)
 
       my_config->racine = racine;
 
+/* load_cat_from_file() is supposed to free the old filename
+ * so we do not keep something pointing to it to free it after
+ */
+      filename = g_strdup (filename);
       if (load_cat_from_file (filename, my_config->racine) == -1)
 	{
 	  return;
@@ -172,7 +173,6 @@ cb_open_recent (GtkWidget * w, gpointer data)
 
       gtk_window_set_title (GTK_WINDOW (main_window), filename);
       menus_set_sensitive (TRUE);
-
     }
 }
 
