@@ -141,7 +141,7 @@ del_the_categorie (GtkWidget * w, FOLDER * racine)
     {
       if (yes_no_cancel_dialog
 	  (_("Delete confirmation"),
-	   _("Do you really want to delete this category ?")) == 0)
+	   _("Do you really want to delete this category?")) == 0)
 	{
 	  gtk_clist_get_text ((GtkCList *) cat_clist, selected_row, 0,
 			      &(tmp[0]));
@@ -159,7 +159,7 @@ del_the_categorie (GtkWidget * w, FOLDER * racine)
 	      if (fd->categorie > id_cat + 1)
 		fd->categorie--;
 	    }
-	  list_folder (racine->tree);
+	  list_folder (racine->tree, TRUE);
 	  racine->is_modified = TRUE;
 	}
     }
@@ -296,7 +296,7 @@ void
 remove_unused_clicked (GtkWidget * w, FOLDER * racine)
 {
   remove_unused_categories (racine);
-  list_folder (racine->tree);
+  list_folder (racine->tree, TRUE);
 
 
 }
@@ -368,7 +368,7 @@ edit_categories (FOLDER * racine)
 
   gtk_window_set_title (GTK_WINDOW (cat_window), _("Category editor"));
   gtk_signal_connect (GTK_OBJECT (cat_window),
-		      "delete_event", destroy_window, NULL);
+		      "delete_event", destroy_window, racine);
   gtk_window_set_policy (GTK_WINDOW (cat_window), FALSE, TRUE, TRUE);
 
   gtk_widget_show (cat_window);
@@ -435,14 +435,14 @@ edit_categories (FOLDER * racine)
   gtk_widget_show (hbox1);
 
 
-  button_import = gnome_stock_or_ordinary_button ("Import");
+  button_import = gtk_button_new_with_label (_("Import"));
   gtk_box_pack_start (GTK_BOX (hbox1), button_import, TRUE, TRUE, 0);
   gtk_widget_show (button_import);
   gtk_signal_connect (GTK_OBJECT (button_import), "clicked",
 		      GTK_SIGNAL_FUNC (import_clicked), racine);
 
 
-  button_remove_unused = gnome_stock_or_ordinary_button ("Remove unused");
+  button_remove_unused = gtk_button_new_with_label (_("Remove unused"));
   gtk_box_pack_start (GTK_BOX (hbox1), button_remove_unused, TRUE, TRUE, 0);
   gtk_widget_show (button_remove_unused);
   gtk_signal_connect (GTK_OBJECT (button_remove_unused), "clicked",
@@ -456,7 +456,7 @@ edit_categories (FOLDER * racine)
 		      GTK_SIGNAL_FUNC (add_the_categorie), racine);
 
 
-  button_edit = gnome_stock_or_ordinary_button ("Edit");
+  button_edit = gtk_button_new_with_label (_("Edit"));
   gtk_box_pack_start (GTK_BOX (vbox1), button_edit, TRUE, FALSE, 0);
   gtk_widget_show (button_edit);
   gtk_signal_connect (GTK_OBJECT (button_edit), "clicked",
