@@ -38,6 +38,9 @@
 #include <errno.h>
 #include <unistd.h>
 #include <string.h>
+#if defined(OS_LINUX)
+#    include <linux/cdrom.h>
+#endif
 
 #include "addisk.h"
 #include "config_common.h"
@@ -59,6 +62,72 @@ GtkWidget *name_entry;
 static pthread_t thread1;
 #endif
 
+/*
+ * Takes a device name for input, and returns its volume name.
+ * allocates 21 bytes for the return buffer.
+ */
+char *
+get_disk_label (char *dev_name)
+{
+#if defined(OS_LINUX)
+  struct stat statbuf;
+  FILE *inf;
+  int i;
+
+  if (stat (dev_name, &statbuf))
+    return NULL;
+
+  if (S_ISREG (statbuf.st_mode))
+    return NULL;
+
+  if ((inf = fopen (dev_name, "rb")))
+    {
+      char *buf;
+      int was_read = 0;
+      unsigned char buf2[32809];
+		/* (comments by David Erman <di97der@student.hk-r.se>)
+		 * Why 32808 bytes? Well, after doing a:
+		 * 
+		 *  dd count=70 if=/dev/cdrom
+		 * 
+		 * that's where I found the title of the CD
+		 *  basically. I couldn't find any docs on the
+		 *  ISO9660 filesystem, so I'm not sure why this is
+		 *  but it seems to be a consistent value. Looks
+		 *  like it might be the fs header there, since
+		 *  there is info on the creator and such as well.
+		 *  
+		 *  Thus, we fseek 32808 bytes, which will amount to
+		 *  16 2048-byte blocks; 2048 byte blocks are used on
+		 *  ISO9660 CD's right?
+		 *
+		 *  I'm not sure this'll work on HFS CD's, but there you go.
+		 */
+
+      fread (buf2, 4101, 8, inf);
+      buf = (char *) g_malloc (MAX_DISK_LABEL_LENGTH+1);
+      was_read = fread (buf, MAX_DISK_LABEL_LENGTH, 1, inf);
+
+      fclose (inf);
+
+      i = 0;
+      while (i++ < (MAX_DISK_LABEL_LENGTH))
+	if (buf[i] == ' ')
+	  break;
+
+      buf[i] = 0;
+      return buf;
+    }
+  else
+    {
+      return NULL;
+    }
+#else
+  return NULL;
+#endif
+}
+
+
 int
 fad_create_window (gchar ** fast_name)
 {
@@ -66,6 +135,8 @@ fad_create_window (gchar ** fast_name)
   GtkWidget *frame;
   int button;
 
+  if ((my_config->read_cd_name) && (!my_config->validate_cd_name))
+    return (1);
   /* Window */
   name_window = gnome_dialog_new (_("Set disk name"),
 				  GNOME_STOCK_BUTTON_OK,
@@ -96,9 +167,9 @@ fad_create_window (gchar ** fast_name)
   gtk_container_add (GTK_CONTAINER (frame), name_entry);
   gtk_widget_show (name_entry);
   gtk_widget_grab_focus (name_entry);
-  gnome_dialog_set_default(GNOME_DIALOG(name_window), GNOME_OK);
-  gnome_dialog_editable_enters(GNOME_DIALOG(name_window), 
-      GTK_EDITABLE(name_entry));
+  gnome_dialog_set_default (GNOME_DIALOG (name_window), GNOME_OK);
+  gnome_dialog_editable_enters (GNOME_DIALOG (name_window),
+				GTK_EDITABLE (name_entry));
 
   button = gnome_dialog_run (GNOME_DIALOG (name_window));
 
@@ -138,14 +209,31 @@ fast_add_disk (FOLDER * racine)
 
   /* Fetch the content of the drive meanwhile */
   do_not_forget_to_mount (TRUE);
-  set_local_disk_name (g_strdup (""));
+  if ((my_config->mount_device) && (my_config->read_cd_name))
+    {
+      if (my_config->fast_name)
+	{
+	  gchar *tmp;
+	  tmp = get_disk_label (my_config->mount_device->str);
+	  if (tmp)
+	    {
+	      free (my_config->fast_name);
+	      my_config->fast_name = tmp;
+	      set_local_disk_name (my_config->fast_name);
+	    }
+	  else
+	    {
+	      set_local_disk_name (g_strdup (""));
+	    }
+	}
+    }
   set_local_folder_name (escapespaces (my_config->mount_point->str));
 
   set_thread_status_to_running ();
 //  progress_timeout (NULL);
 //  do_not_forget_to_umount (TRUE);
   init_vfs_scan_status ();
-  set_added_disk(ADDED_DISK_UNDEF);
+  set_added_disk (ADDED_DISK_UNDEF);
 #if defined(USE_PTHREADS)
   if (pthread_create (&thread1, NULL, start_thread_scan, racine) < 0)
     {
