@@ -21,19 +21,7 @@
 
 #include "config_common.h"
 
-gboolean eject_disk(CONFIG *my_config);
-gboolean mount_disk(CONFIG *my_config);
-gboolean umount_disk(CONFIG *my_config);
-gboolean check_supermount(CONFIG *my_config);
-void *start_fast_add_disk(void *arg);
-void get_name(GtkWidget *w, gpointer data);
-void fad_create_window(CONFIG *my_config);
-void fast_add_disk(GtkWidget *w, CONFIG *my_config);
-
-#define FAD_ERROR  0
-#define FAD_MOUNT  1
-#define FAD_SCAN   2
-#define FAD_UMOUNT 3
-#define FAD_DONE   4
+int fad_create_window (gchar **fast_name);
+void fast_add_disk(FOLDER *racine);
 
 #endif
