@@ -15,10 +15,22 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-#ifndef JE_CLICK_SUR_ON_DANS_MENU_POMME_H__
-#define JE_CLICK_SUR_ON_DANS_MENU_POMME_H__
+#ifndef __LOADCAT_H__
+#define __LOADCAT_H__
+#include <zlib.h>
 
-int load_cat_from_file(char *filename);
+#include "config_common.h"
 
+#ifdef CATALOG2
+	#include "load_oldcat.h"
+#endif
+
+GString *read_gstring(gzFile fh);
+FILE_DATA *read_filedata(gzFile gzfh);
+void load_folder_tree(FOLDER *racine, gzFile gzcatalog);
+GPtrArray *load_gptrarray_of_gstrings(gzFile gzfh);
+gboolean update_vfs_fd (GNode * gn, gpointer data);
+int load_cat_from_file(char *filename, FOLDER *racine);
+int load_only_categories_from_file (char *filename, FOLDER * racine);
 
 #endif
