@@ -55,9 +55,9 @@ getStats (char *foldername)
 }
 
 void
-progress_createTimeout ()
+progress_createTimeout (gpointer pstru)
 {
-  gtk_timeout_add (500, progress_timeout, NULL);
+  gtk_timeout_add (PROGRESS_BAR_TIMEOUT, progress_timeout, pstru);
 }
 
 /* Update the progress_bar.
