@@ -121,6 +121,8 @@ static gboolean SEARCH_OPTIONS_search_in_vfs;
 
 static gboolean gc_descriptions_flag;
 
+static FOLDER *local_racine;
+
 GNode *
 find_node_with_parent (GNode * parent, gchar * name,
 		       gboolean case_sensitive, FOLDER * racine)
@@ -322,14 +324,9 @@ find_files (GNode * gn, gpointer data)
   return (FALSE);
 }
 
-gboolean
-simply_find_files (GNode * gn, gpointer data)
+gboolean find_files_copy (GNode * gn, gpointer data)
 {
-  FOLDER *racine = data;
-  const gchar *result[8];
-  gint row;
-  PIX *pix;
-  gchar *nothing = " ";
+  GPtrArray *search_racine = data;
   FILE_DATA *fd;
 
   progress_setCurrent (progress_getCurrent (GNOMEAPPBAR_SEARCH) + 1,
@@ -344,6 +341,9 @@ simply_find_files (GNode * gn, gpointer data)
     return (FALSE);
 
   fd = get_file_data_from_gnode (gn);
+  if ((!gtk_toggle_button_get_active
+       (GTK_TOGGLE_BUTTON (display_dirs))) && (is_dir (gn)))
+    return (FALSE);
   if (((fd->a_parent_node_is_vfs != IS_VFS)
        && (fd->a_parent_node_is_vfs != IS_VFS_EXTENDED))
       || SEARCH_OPTIONS_search_in_vfs == TRUE)
@@ -353,42 +353,31 @@ simply_find_files (GNode * gn, gpointer data)
 	  (SEARCH_OPTIONS_filename, fd->name->str, SEARCH_OPTIONS_file_regex,
 	   SEARCH_OPTIONS_file_casesensitive)) return (FALSE);
 
-      result[0] = fd->name->str;
-      result[1] = get_disk_from_node (gn);
-      result[2] = get_path_from_node (gn)->str;
-      if (is_dir (gn) == TRUE)
-	result[3] = g_strdup ("0");
-      else
-	result[3] = get_size (fd->taille);
-      result[4] = get_time (fd->date, my_config->date_format);
-      result[5] = get_categorie (racine, fd->categorie);
-      if (!result[5])
-	result[5] = nothing;
-      result[6] = get_description (racine, fd->description);
-      if (!result[6])
-	result[6] = nothing;
-      result[7] = folder_get_informations (fd->node);
-      if ((!result[7]) || (fd->a_parent_node_is_vfs == IS_VFS))
-	result[7] = nothing;
-
-      row = gtk_clist_append (GTK_CLIST (search_list), (gchar **) result);
-      /* pixmap depending on the extension */
-
-      pix = get_pix_from_gnode (gn);
-
-      gtk_clist_set_pixtext (GTK_CLIST (search_list), row, 0,
-			     fd->name->str, 3, pix->pixmap, *(pix->mask));
-
-      gtk_clist_set_row_data (GTK_CLIST (search_list), row, gn);
+      g_ptr_array_add (search_racine, gn);
 
     }
   return (FALSE);
 }
 
 void
-start_search (FOLDER * racine, GNodeTraverseFunc cb_find_files)
+start_search (FOLDER * racine, gboolean is_search_extended)
 {
 
+  GPtrArray *search_racine;
+  gint i, n;
+  gint nc;
+  GNode *gn;
+  FILE_DATA *fd;
+  GString *current_filepath;
+  const gchar *tmpstr;
+  gchar *gs_cat_replace, *gs_desc_replace;
+  const gchar *result[8];
+  gint row;
+  PIX *pix;
+  gchar *nothing = " ";
+
+
+  search_racine = g_ptr_array_new ();
   set_thread_status_to_running ();
 
   gtk_clist_clear (GTK_CLIST (search_list));
@@ -404,8 +393,281 @@ start_search (FOLDER * racine, GNodeTraverseFunc cb_find_files)
   progress_setStatus (_("Searching..."), GNOMEAPPBAR_SEARCH);
   progress_createTimeout (GINT_TO_POINTER (GNOMEAPPBAR_SEARCH));
 
+  local_racine = racine;
   g_node_traverse (SEARCH_OPTIONS_search_root, G_PRE_ORDER, G_TRAVERSE_ALL,
-		   -1, cb_find_files, racine);
+		   -1, find_files_copy, search_racine);
+
+  n = search_racine->len;
+  for (i = 0; i < n; i++)
+    {
+      gn = g_ptr_array_index (search_racine, i);
+      fd = get_file_data_from_gnode (gn);
+      /* Compare paths */
+      current_filepath = get_path_from_node (gn);
+      if (compare
+	  (SEARCH_OPTIONS_filepath, current_filepath->str,
+	   SEARCH_OPTIONS_path_regex, SEARCH_OPTIONS_path_casesensitive))
+	{
+	  g_ptr_array_index (search_racine, i) = NULL;
+	}
+      g_string_free (current_filepath, TRUE);
+    }
+  if (is_search_extended)
+    {
+      /* Compare categories */
+      if (strcmp (SEARCH_OPTIONS_category, STRING_ALL))
+	{
+	  for (i = 0; i < n; i++)
+	    {
+	      gn = g_ptr_array_index (search_racine, i);
+	      if (gn)
+		{
+		  fd = get_file_data_from_gnode (gn);
+		  tmpstr = get_categorie (racine, fd->categorie);
+		  if (tmpstr)
+		    {
+		      if (compare
+			  (SEARCH_OPTIONS_category, tmpstr, COMPARE_EXACT,
+			   FALSE))
+			g_ptr_array_index (search_racine, i) = NULL;
+		    }
+		  else
+		    g_ptr_array_index (search_racine, i) = NULL;
+		}
+	    }
+	}
+      /* Compare descriptions */
+      if (strcmp (SEARCH_OPTIONS_description, ".*"))
+	{
+	  for (i = 0; i < n; i++)
+	    {
+	      gn = g_ptr_array_index (search_racine, i);
+	      if (gn)
+		{
+		  fd = get_file_data_from_gnode (gn);
+		  tmpstr = get_description (racine, fd->description);
+		  if (compare
+		      (SEARCH_OPTIONS_description, tmpstr,
+		       SEARCH_OPTIONS_desc_regex,
+		       SEARCH_OPTIONS_desc_casesensitive))
+		    g_ptr_array_index (search_racine, i) = NULL;
+		}
+	    }
+	}
+
+      /* Compare information */
+      if (strcmp (SEARCH_OPTIONS_information, ".*"))
+	{
+	  for (i = 0; i < n; i++)
+	    {
+	      gn = g_ptr_array_index (search_racine, i);
+	      if (gn)
+		{
+		  fd = get_file_data_from_gnode (gn);
+		  tmpstr = folder_get_information_from_file_data (fd);
+		  if (compare
+		      (SEARCH_OPTIONS_information, tmpstr,
+		       SEARCH_OPTIONS_info_regex,
+		       SEARCH_OPTIONS_info_casesensitive))
+		    g_ptr_array_index (search_racine, i) = NULL;
+		}
+	    }
+	}
+      /* Compare date */
+      if (SEARCH_OPTIONS_size1)
+	{
+	  for (i = 0; i < n; i++)
+	    {
+	      gn = g_ptr_array_index (search_racine, i);
+	      if (gn)
+		{
+		  fd = get_file_data_from_gnode (gn);
+		  if (fd->taille < SEARCH_OPTIONS_size_val1)
+		    g_ptr_array_index (search_racine, i) = NULL;
+		}
+	    }
+	}
+      if (SEARCH_OPTIONS_size2)
+	{
+	  for (i = 0; i < n; i++)
+	    {
+	      gn = g_ptr_array_index (search_racine, i);
+	      if (gn)
+		{
+		  fd = get_file_data_from_gnode (gn);
+		  if (fd->taille > SEARCH_OPTIONS_size_val2)
+		    g_ptr_array_index (search_racine, i) = NULL;
+		}
+	    }
+	}
+
+      /* Compare size */
+      if (SEARCH_OPTIONS_date1)
+	{
+	  for (i = 0; i < n; i++)
+	    {
+	      gn = g_ptr_array_index (search_racine, i);
+	      if (gn)
+		{
+		  fd = get_file_data_from_gnode (gn);
+		  if (fd->date < date_val1)
+		    g_ptr_array_index (search_racine, i) = NULL;
+		}
+	    }
+	}
+      if (SEARCH_OPTIONS_date2)
+	{
+	  for (i = 0; i < n; i++)
+	    {
+	      gn = g_ptr_array_index (search_racine, i);
+	      if (gn)
+		{
+		  fd = get_file_data_from_gnode (gn);
+		  if (fd->date > date_val2)
+		    g_ptr_array_index (search_racine, i) = NULL;
+		}
+	    }
+	}
+
+
+      /* Replace ? */
+
+      if (gtk_toggle_button_get_active
+	  (GTK_TOGGLE_BUTTON (cat_replace_button)))
+	{
+	  gs_cat_replace =
+	    g_strdup (gtk_entry_get_text
+		      (GTK_ENTRY (cat_replace_combo_entry)));
+	  nc = string_to_index (racine->categories, gs_cat_replace);
+	  for (i = 0; i < n; i++)
+	    {
+	      gn = g_ptr_array_index (search_racine, i);
+	      if (gn)
+		{
+		  fd = get_file_data_from_gnode (gn);
+		  if (strcmp (gs_cat_replace, _(STRING_NONE)))
+		    fd->categorie = nc;
+		  else
+		    fd->categorie = 0;
+		}
+	    }
+	}
+      if (gtk_toggle_button_get_active
+	  (GTK_TOGGLE_BUTTON (desc_replace_button)))
+	{
+	  gs_desc_replace =
+	    g_strdup (gtk_entry_get_text (GTK_ENTRY (desc_replace_entry)));
+	  nc = description_add (racine->descriptions, gs_desc_replace);
+	  if (strlen (gs_desc_replace))
+	    {
+	      for (i = 0; i < n; i++)
+		{
+		  gn = g_ptr_array_index (search_racine, i);
+		  if (gn)
+		    {
+		      fd = get_file_data_from_gnode (gn);
+		      fd->description = nc;
+		    }
+		}
+	    }
+	  else
+	    {
+	      for (i = 0; i < n; i++)
+		{
+		  gn = g_ptr_array_index (search_racine, i);
+		  if (gn)
+		    {
+		      fd = get_file_data_from_gnode (gn);
+		      fd->description = 0;
+		    }
+		}
+	      gc_descriptions_flag = TRUE;
+	    }
+	}
+
+      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (info2cat_button)))
+	{
+	  for (i = 0; i < n; i++)
+	    {
+	      gn = g_ptr_array_index (search_racine, i);
+	      if (gn)
+		{
+		  fd = get_file_data_from_gnode (gn);
+		  if (folder_get_informations (fd->node))
+		    {
+		      nc =
+			string_to_index (racine->categories,
+					 g_strdup (folder_get_informations
+						   (fd->node)));
+		      fd = get_file_data_from_gnode (gn);
+		      fd->description = nc;
+		    }
+		}
+	    }
+	}
+
+      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (info2desc_button)))
+	{
+	  for (i = 0; i < n; i++)
+	    {
+	      gn = g_ptr_array_index (search_racine, i);
+	      if (gn)
+		{
+		  fd = get_file_data_from_gnode (gn);
+		  if (folder_get_informations (fd->node))
+		    {
+		      nc =
+			description_add (racine->descriptions,
+					 g_strdup (folder_get_informations
+						   (fd->node)));
+		      fd->description = nc;
+		    }
+		}
+	    }
+	}
+    }
+
+  for (i = 0; i < n; i++)
+    {
+      gn = g_ptr_array_index (search_racine, i);
+      if (gn)
+	{
+	  fd = get_file_data_from_gnode (gn);
+	  result[0] = fd->name->str;
+	  result[1] = get_disk_from_node (gn);
+	  result[2] = get_path_from_node (gn)->str;
+	  if (is_dir (gn) == TRUE)
+	    result[3] = g_strdup ("0");
+	  else
+	    result[3] = get_size (fd->taille);
+	  result[4] = get_time (fd->date, my_config->date_format);
+	  result[5] = get_categorie (racine, fd->categorie);
+	  if (!result[5])
+	    result[5] = nothing;
+	  result[6] = get_description (racine, fd->description);
+	  if (!result[6])
+	    result[6] = nothing;
+	  result[7] = folder_get_informations (fd->node);
+	  if ((!result[7]) || (fd->a_parent_node_is_vfs == IS_VFS))
+	    result[7] = nothing;
+
+	  row = gtk_clist_append (GTK_CLIST (search_list), (gchar **) result);
+	  /* pixmap depending on the extension */
+
+	  pix = get_pix_from_gnode (gn);
+
+	  gtk_clist_set_pixtext (GTK_CLIST (search_list), row, 0,
+				 fd->name->str, 3, pix->pixmap, *(pix->mask));
+
+	  gtk_clist_set_row_data (GTK_CLIST (search_list), row, gn);
+
+	}
+    }
+  for (i = 0; i < n; i++)
+    {
+      g_ptr_array_index (search_racine, i) = NULL;
+    }
+  g_ptr_array_free (search_racine, FALSE);
 
   if (gc_descriptions_flag == TRUE)
     description_garbage_collection ();
@@ -704,7 +966,7 @@ start_extended_search (GtkWidget * w, gpointer data)
     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkvfs));
 
   gtk_clist_freeze (GTK_CLIST (search_list));
-  start_search (racine, find_files);
+  start_search (racine, TRUE);
   gtk_clist_thaw (GTK_CLIST (search_list));
 
   g_free (SEARCH_OPTIONS_filename);
@@ -755,7 +1017,7 @@ start_simple_search (GtkWidget * w, gpointer data)
   SEARCH_OPTIONS_search_root = racine->tree;
 
   gtk_clist_freeze (GTK_CLIST (search_list));
-  start_search (racine, simply_find_files);
+  start_search (racine, FALSE);
   gtk_clist_thaw (GTK_CLIST (search_list));
 
   g_free (SEARCH_OPTIONS_information);
@@ -1195,12 +1457,12 @@ create_search_dialog (FOLDER * racine)
   if (my_config->previous_search_mode == EVERYTHING_MATCHES)
     {
       gtk_entry_set_editable (GTK_ENTRY (es_entry), FALSE);
-	  gtk_widget_set_sensitive (GTK_WIDGET (es_entry), FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (es_entry), FALSE);
     }
   else
     {
       gtk_entry_set_editable (GTK_ENTRY (es_entry), TRUE);
-	  gtk_widget_set_sensitive (GTK_WIDGET (es_entry), TRUE);
+      gtk_widget_set_sensitive (GTK_WIDGET (es_entry), TRUE);
     }
 
   s_path_tmenu = gtk_option_menu_new ();
@@ -1229,7 +1491,7 @@ create_search_dialog (FOLDER * racine)
 		    (GtkAttachOptions) (0), 0, 0);
 
   gtk_entry_set_editable (GTK_ENTRY (s_path_entry), FALSE);
-	  gtk_widget_set_sensitive (GTK_WIDGET (s_path_entry), FALSE);
+  gtk_widget_set_sensitive (GTK_WIDGET (s_path_entry), FALSE);
 
   label = gtk_label_new (_("Name and location"));
   gtk_widget_show (label);
@@ -1400,7 +1662,7 @@ create_search_dialog (FOLDER * racine)
 		    (GtkAttachOptions) (0), 0, 0);
 
   gtk_entry_set_editable (GTK_ENTRY (desc_entry), FALSE);
-	  gtk_widget_set_sensitive (GTK_WIDGET (desc_entry), TRUE);
+  gtk_widget_set_sensitive (GTK_WIDGET (desc_entry), TRUE);
 
   info_tmenu = gtk_option_menu_new ();
   gtk_widget_show (info_tmenu);
@@ -1427,7 +1689,7 @@ create_search_dialog (FOLDER * racine)
 		    (GtkAttachOptions) (0), 0, 0);
 
   gtk_entry_set_editable (GTK_ENTRY (information_entry), FALSE);
-	  gtk_widget_set_sensitive (GTK_WIDGET (information_entry), TRUE);
+  gtk_widget_set_sensitive (GTK_WIDGET (information_entry), TRUE);
 
   label = gtk_label_new (_("Description and Category"));
   gtk_widget_show (label);
