@@ -58,6 +58,7 @@ start_add_disk (char *foldername, char *diskname, FOLDER * racine)
   progress_timeout (NULL);
   do_not_forget_to_umount (FALSE);
   init_vfs_scan_status ();
+  set_added_disk (ADDED_DISK_OK);
 #if defined(USE_PTHREADS)
   if (pthread_create (&thread1, NULL, start_thread_scan, racine) < 0)
     {
@@ -67,11 +68,9 @@ start_add_disk (char *foldername, char *diskname, FOLDER * racine)
     }
   else
     {
-      set_added_disk (TRUE);
       my_timeout (racine);
     }
 #else
-  set_added_disk (TRUE);
   my_timeout (racine);
   start_thread_scan (racine);
 #endif
